unit hiImg_TextInRect;

interface

{$I share.inc}

uses Windows,Kol,Share,Img_Draw;

type
  THIImg_TextInRect= class(THIDraw2PR)
   private
    GFont   : PGraphicTool;
    procedure SetNewFont(Value:TFontRec);
    function GetTextSize(Rect:Trect; s: string; flag: Cardinal):integer;
   public
    _prop_OffsetLeft: integer;
    _prop_OffsetRight: integer;
    _prop_OffsetTop: integer;
    _prop_OffsetBottom: integer;        
    _prop_AlignHorizon: integer;
    _prop_AlignVertical: integer;
    _prop_WordBreak: integer;
    _prop_Ellipsis: integer;
    _prop_RtlReading: integer;
    _prop_NoPrefix: integer;    
    _prop_SingleLine: integer; 
    
    _event_onDraw: THI_Event;
    _event_onTextRect: THI_Event;
            
    property _prop_Font:TFontRec write SetNewFont;

    destructor Destroy; override;
    procedure _work_doDraw(var _Data:TData; Index:word);

    procedure _work_doFont(var _Data:TData; Index:word);
    procedure _work_doOffsetLeft(var _Data:TData; Index:word);
    procedure _work_doOffsetRight(var _Data:TData; Index:word);
    procedure _work_doOffsetTop(var _Data:TData; Index:word);
    procedure _work_doOffsetBottom(var _Data:TData; Index:word);
    procedure _work_doAlignHorizon(var _Data:TData; Index:word);
    procedure _work_doAlignVertical(var _Data:TData; Index:word);
    procedure _work_doWordBreak(var _Data:TData; Index:word);
    procedure _work_doEllipsis(var _Data:TData; Index:word);
    procedure _work_doRtlReading(var _Data:TData; Index:word);
    procedure _work_doNoPrefix(var _Data:TData; Index:word);
    procedure _work_doSingleLine(var _Data:TData; Index:word);

  end;

implementation

destructor THIImg_TextInRect.Destroy;
begin
   GFont.free;
   inherited;
end;

function THIImg_TextInRect.GetTextSize;
var  DC: HDC;
begin
   DC := CreateCompatibleDC(0);
   SelectObject(DC, GFont.Handle);
   Result:= DrawText(DC,PChar(s), -1, Rect, flag);  
   DeleteDC(DC);
end;

procedure THIImg_TextInRect._work_doDraw;
var   hOldFont: HFONT;
      OldFontSize: Integer;
      s:string;
      flag: Cardinal;
      SizeRect, Rect: TRect;
      dTop, dWidth, dHeight, Data: TData;
begin
TRY
   if not ImgGetDC(_Data) then pDC := CreateCompatibleDC(0); //exit;
   ReadXY(_Data);
   ImgNewSizeDC;
   s := ReadString(_Data,_data_Text,_prop_Text);
   SetBkMode(pDC, TRANSPARENT);
   SetTextColor(pDC, Color2RGB(GFont.Color));   
   OldFontSize := GFont.FontHeight;
   GFont.FontHeight := Round(GFont.FontHeight * fScale.y);
   hOldFont := SelectObject(pDC, GFont.Handle);

   flag := 0;
   SetRect(Rect, x1 + _prop_OffsetLeft, y1 + _prop_OffsetTop,
                 x2 - _prop_OffsetRight, y2 - _prop_OffsetBottom);

   if _prop_WordBreak = 1 then flag := flag OR DT_WORDBREAK; 
   case _prop_Ellipsis of
    1: flag := flag OR DT_WORD_ELLIPSIS;
    2: flag := flag OR DT_PATH_ELLIPSIS;
    3: flag := flag OR DT_END_ELLIPSIS;
   end;
   if _prop_RtlReading = 1 then flag := flag OR DT_RTLREADING; 
   if _prop_NoPrefix = 1 then flag := flag OR DT_NOPREFIX;
   if _prop_SingleLine = 1 then flag := flag OR DT_SINGLELINE;

   case _prop_AlignHorizon of
    0: flag := flag OR DT_LEFT;
    1: flag := flag OR DT_RIGHT;
    2: flag := flag OR DT_CENTER; 
   end; 
   case _prop_AlignVertical of
    0: flag := flag OR DT_TOP;
    1: begin
       flag := flag OR DT_TOP;
       OffsetRect(Rect,0, Rect.Bottom - GetTextSize(Rect,s,flag) - Rect.Top);
       end;
    2: begin
       flag := flag OR DT_TOP;
       OffsetRect(Rect,0, Rect.Bottom div 2 - GetTextSize(Rect,s,flag) div 2 - Rect.Top div 2);
       end;
   end; 
   SizeRect := Rect;
   DrawText(pDC,PChar(s), -1, SizeRect, flag OR DT_CALCRECT);  
   if _prop_AlignHorizon = 1 then begin
    SizeRect.Left := SizeRect.Left + Rect.Right - SizeRect.Right;
    SizeRect.Right := SizeRect.Right + Rect.Right - SizeRect.Right;
   end;
   dtInteger(Data, SizeRect.Left);
   dtInteger(dWidth, SizeRect.Right);
   dtInteger(dTop, SizeRect.Top);
   dtInteger(dHeight, SizeRect.Bottom);
   Data.ldata:= @dTop;
   dTop.ldata:= @dWidth;
   dWidth.ldata:= @dHeight;
   _hi_OnEvent(_event_onTextRect, Data);

   DrawText(pDC,PChar(s), -1, Rect, flag);
 
   SelectObject(pDC, hOldFont);
   GFont.FontHeight := OldFontSize;
FINALLY
   ImgReleaseDC;
   _hi_OnEvent(_event_onDraw);
END;
end;

procedure THIImg_TextInRect._work_doFont;
begin
   if _IsFont(_Data) then SetNewFont(PFontRec(_Data.idata)^);
end;

procedure THIImg_TextInRect._work_doAlignHorizon;
begin
   _prop_AlignHorizon := ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doAlignVertical;
begin
   _prop_AlignVertical := ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doWordBreak;
begin
   _prop_WordBreak := ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doEllipsis;
begin
   _prop_Ellipsis := ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doRtlReading;
begin
   _prop_RtlReading := ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doNoPrefix;
begin
   _prop_NoPrefix := ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doSingleLine;
begin
   _prop_SingleLine := ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doOffsetLeft;
begin
   _prop_OffsetLeft:= ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doOffsetRight;
begin
   _prop_OffsetRight:= ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doOffsetTop;
begin
   _prop_OffsetTop:= ToInteger(_Data);
end;

procedure THIImg_TextInRect._work_doOffsetBottom;
begin
   _prop_OffsetBottom:= ToInteger(_Data);
end;

procedure THIImg_TextInRect.SetNewFont;
begin
   if Assigned(GFont) then GFont.free;
   GFont := NewFont;
   GFont.Color:= Value.Color;
   Share.SetFont(GFont,Value.Style);
   GFont.FontName:= Value.Name;
   GFont.FontHeight:= _hi_SizeFnt(Value.Size);
   GFont.FontCharset:= Value.CharSet;
end;

end.